@@ -1,7 +1,3 @@
-#!/usr/bin/perl
-
-=pod
-
 =head1 NAME
 
 Nagios::NRPE::Packet - Assembly and disassembly of an NRPE packet
@@ -95,10 +91,6 @@ A helper function to assemble a V2 packet.
 
 A helper function to assemble a V3 packet.
 
-=item deassemble()
-
-Deprecated function, use disassemble instead.
-
 =item disassemble()
 
 Takes a packet recieved by either client or server and disassembles them. The returned hashref contains 
@@ -194,13 +186,14 @@ the same terms as the Perl 5 programming language system itself.
 
 package Nagios::NRPE::Packet;
 
-our $VERSION = '1.0.3';
+our $VERSION = '2.0.1';
 
 use 5.010_000;
 require Exporter;
 require overload;
 
-BEGIN {
+BEGIN
+{
     @ISA       = qw(Exporter);
     @EXPORT_OK = qw(NRPE_PACKET_VERSION_3
       NRPE_PACKET_VERSION_2
@@ -251,43 +244,62 @@ use constant {
     STATE_OK       => 0,
 };
 
-sub new {
-    my ( $class, %options ) = @_;
+sub new
+{
+    my ($class, %options) = @_;
     my $self = {};
 
-    # taken with modifications from common.h in nagios-nrpe
-    my $c = Convert::Binary::C->new( ByteOrder => 'BigEndian', Alignment => 0 );
-    $self->{c} = $c;
     bless $self, $class;
 }
 
-sub assemble {
-    my ( $self, %options ) = @_;
+sub assemble
+{
+    my ($self, %options) = @_;
+
+    # taken with modifications from common.h in nagios-nrpe
+    my $c = Convert::Binary::C->new(ByteOrder => 'BigEndian', Alignment => 0);
+    $self->{c} = $c;
+
     croak "ERROR: Cannot send Packet with empty buffer!"
-      if ( not defined $options{check} );
+      if (not defined $options{check});
     my $packed;
-    if ( $options{version} eq NRPE_PACKET_VERSION_2 ) {
+    if ($options{version} eq NRPE_PACKET_VERSION_2)
+    {
         $packed = $self->assemble_v2(%options);
     }
-    else {
+    else
+    {
         $packed = $self->assemble_v3(%options);
     }
     return $packed;
 
 }
 
-sub assemble_v3 {
-    my ( $self, %options ) = @_;
-    my $unpacked = {};
-    my $len      = length( $options{check} ) + 1;
+sub assemble_v3
+{
+    my ($self, %options) = @_;
+    my $buffer = $options{check};
+    my $len    = length($buffer);
 
+    # In order for crc32 calculation to be correct we need to pad the buffer with \0
+    # It seems that the buffer must be in multiples of 1024 so to achive this we use
+    # some integer arithmetic to find the next multiple of 1024 that can hold our message
+    my $packLen;
+    {
+        use integer;
+        $packLen = (($len / 1024) * 1024) + 1024;
+    }
+    $buffer = pack("Z$packLen", $buffer);
+    $len = length( $buffer) + 1;
+
+    my $unpacked;
     $unpacked->{alignment}      = 0;
     $unpacked->{buffer_length}  = $len;
-    $unpacked->{buffer}         = $options{check};
+    $unpacked->{buffer}         = $buffer;
     $unpacked->{crc32_value}    = "\x00\x00\x00\x00";
-    $unpacked->{packet_type}    = $options{type} || NRPE_PACKET_QUERY;
+    $unpacked->{packet_type}    = $options{type} // NRPE_PACKET_QUERY;
     $unpacked->{packet_version} = NRPE_PACKET_VERSION_3;
-    $unpacked->{result_code}    = $options{result_code} || 2324;
+    $unpacked->{result_code}    = $options{result_code} // 2324;
 
     $self->{c}->parse(<<PACKET_STRUCT);
 struct Packet{
@@ -300,26 +312,27 @@ struct Packet{
   char             buffer[$len];
 };
 PACKET_STRUCT
-    $self->{c}->tag( 'Packet.buffer', Format => 'String' );
+    $self->{c}->tag('Packet.buffer', Format => 'String');
 
-    my $packed = $self->{c}->pack( 'Packet', $unpacked );
+    my $packed = $self->{c}->pack('Packet', $unpacked);
 
     $unpacked->{crc32_value} = crc32($packed);
-    $packed = $self->{c}->pack( 'Packet', $unpacked );
+    $packed = $self->{c}->pack('Packet', $unpacked);
     return $packed;
 
 }
 
-sub assemble_v2 {
+sub assemble_v2
+{
 
-    my ( $self, %options ) = @_;
+    my ($self, %options) = @_;
     my $unpacked = {};
 
     $unpacked->{buffer}         = $options{check};
     $unpacked->{crc32_value}    = "\x00\x00\x00\x00";
-    $unpacked->{packet_type}    = $options{type} || NRPE_PACKET_QUERY;
+    $unpacked->{packet_type}    = $options{type} // NRPE_PACKET_QUERY;
     $unpacked->{packet_version} = NRPE_PACKET_VERSION_2;
-    $unpacked->{result_code}    = $options{result_code} || 2324;
+    $unpacked->{result_code}    = $options{result_code} // 2324;
 
     $self->{c}->parse(<<PACKET_STRUCT);
 struct Packet{
@@ -330,56 +343,86 @@ struct Packet{
   char             buffer[1024];
 };
 PACKET_STRUCT
-    $self->{c}->tag( 'Packet.buffer', Format => 'String' );
+    $self->{c}->tag('Packet.buffer', Format => 'String');
 
-    my $packed = $self->{c}->pack( 'Packet', $unpacked );
+    my $packed = $self->{c}->pack('Packet', $unpacked);
 
     $unpacked->{crc32_value} = crc32($packed);
-    $packed = $self->{c}->pack( 'Packet', $unpacked );
+    $packed = $self->{c}->pack('Packet', $unpacked);
     return $packed;
 
 }
 
-sub validate {
-    my ( $self, $packet ) = @_;
-    my $unpacked = $self->{c}->unpack( 'Packet', $packet );
-    my $checksum = $unpacked->{crc32_value};
-    $unpacked->{crc32_value} = "\x00\x00\x00\x00";
-    my $packed = $self->{c}->pack( 'Packet', $unpacked );
-    if ( crc32($packed) != $checksum ) {
+sub validate
+{
+    my ($self, $packet) = @_;
+    my $unpacked = $self->disassemble($packet, 1);
+    if (!$unpacked->{packet_version})
+    {
+        # If version is missing this is probably not an NRPE Packet.
         return undef;
     }
-    else {
+    my $checksum = $unpacked->{crc32_value};
+    $unpacked->{crc32_value} = "\x00\x00\x00\x00";
+    my $packed = $self->assemble(
+                                 %{
+                                     {
+                                      check   => $unpacked->{buffer},
+                                      version => $unpacked->{packet_version},
+                                      type    => $unpacked->{packet_type},
+                                      result_code => $unpacked->{result_code}
+                                     }
+                                  }
+                                );
+    if (crc32($packed) != $checksum)
+    {
+        return undef;
+    }
+    else
+    {
         return 1;
     }
 }
 
-sub deassemble {
-    my ( $self, $packet ) = @_;
-    print
-"The deassemble function has been deprecated and will be removed in a future release, please use disassemble instead\n";
-    return $self->disassemble($packet);
-}
-
-sub disassemble {
-    my ( $self, $packet ) = @_;
-    if ( !$packet ) {
+sub disassemble
+{
+    my ($self, $packet, $novalidate) = @_;
+    if (!$packet)
+    {
         return_error("Could not disassemble packet, it seems empty");
     }
-    my $ver = unpack( "n", $packet );
+    unless ($novalidate)
+    {
+        unless ($self->validate($packet))
+        {
+            return_error("Packet had invalid CRC32.");
+        }
+    }
+    my $ver = unpack("n", $packet);
     my $unpacked = {};
-    if ( $ver eq NRPE_PACKET_VERSION_2 ) {
-        $unpacked = $self->disassemble_v2($packet);
+    if ($ver)
+    {
+        if ($ver eq NRPE_PACKET_VERSION_2)
+        {
+            $unpacked = $self->disassemble_v2($packet);
+        }
+        else
+        {
+            $unpacked = $self->disassemble_v3($packet);
+        }
     }
-    else {
-        $unpacked = $self->disassemble_v3($packet);
+    else
+    {
+        return undef;
     }
+
     return $unpacked;
 }
 
-sub disassemble_v3 {
-    my ( $self, $packet ) = @_;
-    my @arr = unpack( "n2 N n2 N Z*", $packet );
+sub disassemble_v3
+{
+    my ($self, $packet) = @_;
+    my @arr = unpack("n2 N n2 N Z*", $packet);
     my $unpacked = {};
     $unpacked->{packet_version} = $arr[0];
     $unpacked->{packet_type}    = $arr[1];
@@ -391,9 +434,10 @@ sub disassemble_v3 {
     return $unpacked;
 }
 
-sub disassemble_v2 {
-    my ( $self, $packet ) = @_;
-    my @arr = unpack( "n2 N n Z*", $packet );
+sub disassemble_v2
+{
+    my ($self, $packet) = @_;
+    my @arr = unpack("n2 N n Z*", $packet);
     my $unpacked = {};
     $unpacked->{packet_version} = $arr[0];
     $unpacked->{packet_type}    = $arr[1];
@@ -403,7 +447,8 @@ sub disassemble_v2 {
     return $unpacked;
 }
 
-sub packet_dump {
+sub packet_dump
+{
     my $packet = shift;
     my $i;
     my $k;
@@ -411,27 +456,33 @@ sub packet_dump {
     my $l;
     my $ascii;
     my $c;
-    for ( $i = 0 ; $i < length($packet) ; $i += 16 ) {
+    for ($i = 0 ; $i < length($packet) ; $i += 16)
+    {
         $l = $i + 16;
-        if ( $l > length($packet) ) {
+        if ($l > length($packet))
+        {
             $l = length($packet);
         }
-        $dump = sprintf( "%04d - %04d: ", $i, $l );
+        $dump = sprintf("%04d - %04d: ", $i, $l);
         $ascii = "";
-        for ( $k = $i ; $k < $l ; $k++ ) {
-            $c = ( ord( substr( $packet, $k, 1 ) ) );
-            $dump .= sprintf( "%02x ", $c );
-            if ( ( $c >= 32 ) && ( $c <= 126 ) ) {
+        for ($k = $i ; $k < $l ; $k++)
+        {
+            $c = (ord(substr($packet, $k, 1)));
+            $dump .= sprintf("%02x ", $c);
+            if (($c >= 32) && ($c <= 126))
+            {
                 $ascii .= chr($c);
             }
-            else {
+            else
+            {
                 $ascii .= ".";
             }
         }
-        for ( $k = 0 ; $k < ( $i + 16 - $l ) ; $k++ ) {
+        for ($k = 0 ; $k < ($i + 16 - $l) ; $k++)
+        {
             $dump .= "   ";
         }
-        print( "packet_dump() " . $dump . " [" . $ascii . "]" . "\n" );
+        print("packet_dump() " . $dump . " [" . $ascii . "]" . "\n");
     }
 }
 
